@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2014-2016, Matteo Collina <hello@matteocollina.com>
+Copyright (c) 2014-2018, Matteo Collina <hello@matteocollina.com>
 
 Permission to use, copy, modify, and/or distribute this software for any
 purpose with or without fee is hereby granted, provided that the above
@@ -16,32 +16,45 @@ IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 'use strict'
 
-var through = require('through2')
-var StringDecoder = require('string_decoder').StringDecoder
+const { Transform } = require('readable-stream')
+const { StringDecoder } = require('string_decoder')
+const kLast = Symbol('last')
+const kDecoder = Symbol('decoder')
 
 function transform (chunk, enc, cb) {
-  this._last += this._decoder.write(chunk)
-  if (this._last.length > this.maxLength) {
-    return cb(new Error('maximum buffer reached'))
+  var list
+  if (this.overflow) { // Line buffer is full. Skip to start of next line.
+    var buf = this[kDecoder].write(chunk)
+    list = buf.split(this.matcher)
+
+    if (list.length === 1) return cb() // Line ending not found. Discard entire chunk.
+
+    // Line ending found. Discard trailing fragment of previous line and reset overflow state.
+    list.shift()
+    this.overflow = false
+  } else {
+    this[kLast] += this[kDecoder].write(chunk)
+    list = this[kLast].split(this.matcher)
   }
 
-  var list = this._last.split(this.matcher)
-
-  this._last = list.pop()
+  this[kLast] = list.pop()
 
   for (var i = 0; i < list.length; i++) {
     push(this, this.mapper(list[i]))
   }
 
+  this.overflow = this[kLast].length > this.maxLength
+  if (this.overflow && !this.skipOverflow) return cb(new Error('maximum buffer reached'))
+
   cb()
 }
 
 function flush (cb) {
   // forward any gibberish left in there
-  this._last += this._decoder.end()
+  this[kLast] += this[kDecoder].end()
 
-  if (this._last) {
-    push(this, this.mapper(this._last))
+  if (this[kLast]) {
+    push(this, this.mapper(this[kLast]))
   }
 
   cb()
@@ -90,21 +103,20 @@ function split (matcher, mapper, options) {
       }
   }
 
-  var stream = through(options, transform, flush)
+  options = Object.assign({}, options)
+  options.transform = transform
+  options.flush = flush
+  options.readableObjectMode = true
 
-  // this stream is in objectMode only in the readable part
-  stream._readableState.objectMode = true
+  const stream = new Transform(options)
 
-  // objectMode default hwm is 16 and not 16384
-  if (stream._readableState.highWaterMark && !options.highWaterMark) {
-    stream._readableState.highWaterMark = 16
-  }
-
-  stream._last = ''
-  stream._decoder = new StringDecoder('utf8')
+  stream[kLast] = ''
+  stream[kDecoder] = new StringDecoder('utf8')
   stream.matcher = matcher
   stream.mapper = mapper
   stream.maxLength = options.maxLength
+  stream.skipOverflow = options.skipOverflow
+  stream.overflow = false
 
   return stream
 }
