@@ -35,6 +35,22 @@ test('split two lines on two writes', function (t) {
   input.end()
 })
 
+test('split four lines on three writes', function (t) {
+  t.plan(2)
+
+  var input = split()
+
+  input.pipe(strcb(function (err, list) {
+    t.error(err)
+    t.deepEqual(list, ['hello', 'world', 'bye', 'world'])
+  }))
+
+  input.write('hello\nwor')
+  input.write('ld\nbye\nwo')
+  input.write('rld')
+  input.end()
+})
+
 test('accumulate multiple writes', function (t) {
   t.plan(2)
 
@@ -302,3 +318,46 @@ test('readable highWaterMark', function (t) {
   t.equal(input._readableState.highWaterMark, 16)
   t.end()
 })
+
+test('maxLength < chunk size', function (t) {
+  t.plan(2)
+
+  var input = split({ maxLength: 2 })
+
+  input.pipe(strcb(function (err, list) {
+    t.error(err)
+    t.deepEqual(list, ['a', 'b'])
+  }))
+
+  input.end('a\nb')
+})
+
+test('maximum buffer limit w/skip', function (t) {
+  t.plan(2)
+
+  var input = split({ maxLength: 2, skipOverflow: true })
+
+  input.pipe(strcb(function (err, list) {
+    t.error(err)
+    t.deepEqual(list, ['a', 'b', 'c'])
+  }))
+
+  input.write('a\n123')
+  input.write('456')
+  input.write('789\nb\nc')
+  input.end()
+})
+
+test("don't modify the options object", function (t) {
+  t.plan(2)
+
+  var options = {}
+  var input = split(options)
+
+  input.pipe(strcb(function (err, list) {
+    t.error(err)
+    t.same(options, {})
+  }))
+
+  input.end()
+})
